#pragma once
// This file is auto-generated! Do not edit!

#define ESPHOME_VERSION "dev"

#define USE_API
#define USE_LOGGER
#define USE_BINARY_SENSOR
#define USE_SENSOR
#define USE_SWITCH
#define USE_WIFI
#define USE_STATUS_LED
#define USE_TEXT_SENSOR
#define USE_FAN
#define USE_COVER
#define USE_LIGHT
#define USE_CLIMATE
#define USE_MQTT
#define USE_POWER_SUPPLY
#define USE_HOMEASSISTANT_TIME
#define USE_JSON
#ifdef ARDUINO_ARCH_ESP32
#define USE_ESP32_CAMERA
#endif
#define USE_TIME
#define USE_DEEP_SLEEP
